@@ -1236,6 +1236,7 @@ impl Config {
 
     pub fn is_compatible(&self) -> Result<bool> {
         let result = match self.parsed_version()? {
+            (0, 8, 5) => true,
             (0, 8, 4) => true,
             (0, 8, 3) => true,
             (0, 8, 2) => true,
@@ -1249,7 +1250,8 @@ impl Config {
 
     pub fn upgrade_notification(&self) -> Result<Option<&str>> {
         let result = match self.parsed_version()? {
-            (0, 8, 4) => None,
+            (0, 8, 5) => None,
+            (0, 8, 4) => Some("App version updated. Improved key bindings"),
             (0, 8, 3) => Some("App version updated. Fixed search"),
             (0, 8, 2) => Some("App version updated. Fixed search"),
             (0, 8, 1) => Some("App version updated. Fixed search"),
